const bufferSize = 1024;
export class AudioPlayer {
    constructor(synthesizer, audioContext) {
        this.synthesizer = synthesizer;
        this.audioContext = audioContext;
        this.started = false;
    }
    start() {
        if (this.started) {
            return;
        }
        this.started = true;
        void this.resumeAudioContext();
        this.synthesizer.reset();
        this.createScriptProcessor();
    }
    stop() {
        if (!this.started) {
            return;
        }
        this.started = false;
        this.releaseScriptProcessor();
    }
    createScriptProcessor() {
        this.scriptProcessor = this.audioContext.createScriptProcessor(bufferSize, 1, 1);
        this.scriptProcessor.connect(this.audioContext.destination);
        this.scriptProcessor.addEventListener("audioprocess", (event) => this.audioprocessEventHandler(event));
        this.dummySource = new ConstantSourceNode(this.audioContext);
        this.dummySource.connect(this.scriptProcessor);
        this.scriptProcessor.connect(this.audioContext.destination);
        this.dummySource.start();
    }
    releaseScriptProcessor() {
        this.dummySource.stop();
        this.scriptProcessor.disconnect();
        this.dummySource.disconnect();
    }
    async resumeAudioContext() {
        if (this.audioContext.state == "suspended") {
            await this.audioContext.resume();
        }
    }
    audioprocessEventHandler(event) {
        const buf = event.outputBuffer.getChannelData(0);
        this.synthesizer.synthesize(buf);
    }
}
//# sourceMappingURL=AudioPlayer.js.map