export const palePink = "#FFEEF5";
export class Button {
    constructor(x, y, width, height, text, switchedOn) {
        this.x = x;
        this.y = y;
        this.width = width;
        this.height = height;
        this.text = text;
        this.switchedOn = switchedOn;
    }
    draw(ctx) {
        const radius = 10;
        ctx.save();
        ctx.strokeStyle = palePink;
        ctx.fillStyle = palePink;
        ctx.globalAlpha = 1;
        ctx.lineCap = 'round';
        ctx.lineJoin = 'round';
        ctx.lineWidth = 2 * radius;
        ctx.beginPath();
        ctx.moveTo(this.x + radius, this.y + radius);
        ctx.lineTo(this.x + this.width - radius, this.y + radius);
        ctx.lineTo(this.x + this.width - radius, this.y + this.height - radius);
        ctx.lineTo(this.x + radius, this.y + this.height - radius);
        ctx.closePath();
        ctx.stroke();
        ctx.fill();
        ctx.font = "16px Arial";
        ctx.textAlign = "center";
        if (this.switchedOn) {
            ctx.fillStyle = "orchid";
            ctx.globalAlpha = 0.6;
        }
        else {
            ctx.fillStyle = "white";
            ctx.globalAlpha = 1;
        }
        this.drawText(ctx);
        ctx.restore();
    }
    drawText(ctx) {
        ctx.fillText(this.text, this.x + this.width / 2, this.y + this.height / 2 + 6);
    }
    handleTouchStart(touch) {
        if (touch.x >= this.x && touch.x <= this.x + this.width && touch.y >= this.y && touch.y <= this.y + this.height) {
            this.switchedOn = !this.switchedOn;
        }
    }
}
export function mapDomToCanvasCoordinates(canvas, clientX, clientY) {
    const rect = canvas.getBoundingClientRect();
    const x1 = clientX - rect.left - (canvas.clientLeft || 0);
    const y1 = clientY - rect.top - (canvas.clientTop || 0);
    const x = x1 / rect.width * canvas.width;
    const y = y1 / rect.height * canvas.height;
    return { x, y };
}
//# sourceMappingURL=GuiUtils.js.map