import { Glottis } from "./Glottis";
import { Tract } from "./Tract";
import { TractShaper } from "./TractShaper";
const maxBlockLength = 512;
export class Synthesizer {
    constructor(sampleRate) {
        this.sampleRate = sampleRate;
        this.glottis = new Glottis(sampleRate);
        const tractSampleRate = 2 * sampleRate;
        this.tract = new Tract(this.glottis, tractSampleRate);
        this.tractShaper = new TractShaper(this.tract);
    }
    reset() {
        this.calculateNewBlockParameters(0);
    }
    synthesize(buf) {
        let p = 0;
        while (p < buf.length) {
            const blockLength = Math.min(maxBlockLength, buf.length - p);
            const blockBuf = buf.subarray(p, p + blockLength);
            this.synthesizeBlock(blockBuf);
            p += blockLength;
        }
    }
    synthesizeBlock(buf) {
        const n = buf.length;
        const deltaTime = n / this.sampleRate;
        this.calculateNewBlockParameters(deltaTime);
        for (let i = 0; i < n; i++) {
            const lambda1 = i / n;
            const lambda2 = (i + 0.5) / n;
            const glottalOutput = this.glottis.step(lambda1);
            const vocalOutput1 = this.tract.step(glottalOutput, lambda1);
            const vocalOutput2 = this.tract.step(glottalOutput, lambda2);
            buf[i] = (vocalOutput1 + vocalOutput2) * 0.125;
        }
    }
    calculateNewBlockParameters(deltaTime) {
        this.glottis.adjustParameters(deltaTime);
        this.tractShaper.adjustTractShape(deltaTime);
        this.tract.calculateNewBlockParameters();
    }
}
//# sourceMappingURL=Synthesizer.js.map