import { moveTowards } from "./Utils";
const gridOffset = 1.7;
export class TractShaper {
    constructor(tract) {
        this.movementSpeed = 15;
        this.velumOpenTarget = 0.4;
        this.velumClosedTarget = 0.01;
        this.lastObstruction = -1;
        this.tract = tract;
        this.targetDiameter = new Float64Array(tract.n);
        this.tongueIndex = 12.9;
        this.tongueDiameter = 2.43;
        this.shapeNose(true);
        tract.calculateNoseReflections();
        this.shapeNose(false);
        this.shapeMainTract();
    }
    shapeMainTract() {
        const tract = this.tract;
        for (let i = 0; i < tract.n; i++) {
            const d = this.getRestDiameter(i);
            tract.diameter[i] = d;
            this.targetDiameter[i] = d;
        }
    }
    getRestDiameter(i) {
        const tract = this.tract;
        if (i < 7) {
            return 0.6;
        }
        if (i < tract.bladeStart) {
            return 1.1;
        }
        if (i >= tract.lipStart) {
            return 1.5;
        }
        const t = 1.1 * Math.PI * (this.tongueIndex - i) / (tract.tipStart - tract.bladeStart);
        const fixedTongueDiameter = 2 + (this.tongueDiameter - 2) / 1.5;
        let curve = (1.5 - fixedTongueDiameter + gridOffset) * Math.cos(t);
        if (i == tract.bladeStart - 2 || i == tract.lipStart - 1) {
            curve *= 0.8;
        }
        if (i == tract.bladeStart || i == tract.lipStart - 2) {
            curve *= 0.94;
        }
        return 1.5 - curve;
    }
    adjustTractShape(deltaTime) {
        const tract = this.tract;
        const amount = deltaTime * this.movementSpeed;
        let newLastObstruction = -1;
        for (let i = 0; i < tract.n; i++) {
            const diameter = tract.diameter[i];
            const targetDiameter = this.targetDiameter[i];
            if (diameter <= 0) {
                newLastObstruction = i;
            }
            let slowReturn;
            if (i < tract.noseStart) {
                slowReturn = 0.6;
            }
            else if (i >= tract.tipStart) {
                slowReturn = 1;
            }
            else {
                slowReturn = 0.6 + 0.4 * (i - tract.noseStart) / (tract.tipStart - tract.noseStart);
            }
            tract.diameter[i] = moveTowards(diameter, targetDiameter, slowReturn * amount, 2 * amount);
        }
        if (this.lastObstruction > -1 && newLastObstruction == -1 && tract.noseDiameter[0] < 0.223) {
            this.addTransient(this.lastObstruction);
        }
        this.lastObstruction = newLastObstruction;
        tract.noseDiameter[0] = moveTowards(tract.noseDiameter[0], this.velumTarget, amount * 0.25, amount * 0.1);
    }
    addTransient(position) {
        const tract = this.tract;
        const transient = {
            position: position,
            startTime: tract.time,
            lifeTime: 0.2,
            strength: 0.3,
            exponent: 200
        };
        tract.transients.push(transient);
    }
    shapeNose(velumOpen) {
        const tract = this.tract;
        this.velumTarget = velumOpen ? this.velumOpenTarget : this.velumClosedTarget;
        for (let i = 0; i < tract.noseLength; i++) {
            let diameter;
            const d = 2 * (i / tract.noseLength);
            if (i == 0) {
                diameter = this.velumTarget;
            }
            else if (d < 1) {
                diameter = 0.4 + 1.6 * d;
            }
            else {
                diameter = 0.5 + 1.5 * (2 - d);
            }
            diameter = Math.min(diameter, 1.9);
            tract.noseDiameter[i] = diameter;
        }
    }
}
//# sourceMappingURL=TractShaper.js.map