export function clamp(x, min, max) {
    return (x < min) ? min : (x > max) ? max : x;
}
export function moveTowards(current, target, amountUp, amountDown) {
    return (current < target) ? Math.min(current + amountUp, target) : Math.max(current - amountDown, target);
}
export function getTime() {
    return Date.now() / 1000;
}
function createBiquadIirFilter(b0, b1, b2, a0, a1, a2) {
    const nb0 = b0 / a0;
    const nb1 = b1 / a0;
    const nb2 = b2 / a0;
    const na1 = a1 / a0;
    const na2 = a2 / a0;
    let x1 = 0;
    let x2 = 0;
    let y1 = 0;
    let y2 = 0;
    return (x) => {
        const y = nb0 * x + nb1 * x1 + nb2 * x2 - na1 * y1 - na2 * y2;
        x2 = x1;
        x1 = x;
        y2 = y1;
        y1 = y;
        return y;
    };
}
export function createBandPassFilter(f0, q, sampleRate) {
    const w0 = 2 * Math.PI * f0 / sampleRate;
    const alpha = Math.sin(w0) / (2 * q);
    const b0 = alpha;
    const b1 = 0;
    const b2 = -alpha;
    const a0 = 1 + alpha;
    const a1 = -2 * Math.cos(w0);
    const a2 = 1 - alpha;
    return createBiquadIirFilter(b0, b1, b2, a0, a1, a2);
}
export function createBufferedWhiteNoiseSource(bufferSize) {
    const buf = new Float64Array(bufferSize);
    for (let i = 0; i < bufferSize; i++) {
        buf[i] = 2 * Math.random() - 1;
    }
    let i = 0;
    return () => {
        if (i >= bufferSize) {
            i = 0;
        }
        return buf[i++];
    };
}
export function createFilteredNoiseSource(f0, q, sampleRate, bufferSize) {
    const whiteNoise = createBufferedWhiteNoiseSource(bufferSize);
    const filter = createBandPassFilter(f0, q, sampleRate);
    return () => filter(whiteNoise());
}
//# sourceMappingURL=Utils.js.map